"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _easevalidation = require("easevalidation");

var _path = _interopRequireDefault(require("path"));

var _cliTable = _interopRequireDefault(require("cli-table"));

var _util = require("util");

var _oors = require("oors");

var winston = _interopRequireWildcard(require("winston"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class LoggerModule extends _oors.Module {
  constructor(...args) {
    super(...args);
    this.name = 'oors.logger';

    this.logError = error => this.logger.error(error.message, {
      meta: {
        error
      }
    });
  }

  setup({
    printModules,
    printDependencyGraph,
    printMiddlewares,
    logsDir,
    level: defaultLevel,
    logGqlErrors
  }) {
    const transports = [new winston.transports.File({
      level: 'error',
      filename: _path.default.join(logsDir, 'errors.log'),
      maxsize: 5000000,
      maxFiles: 10,
      tailable: true,
      format: winston.format.combine(winston.format.timestamp(), winston.format.json())
    })];

    if (process.env.NODE_ENV !== 'production') {
      transports.push(new winston.transports.Console({
        format: winston.format.combine(winston.format.timestamp(), winston.format.colorize(), winston.format.simple())
      }));
    }

    const logger = this.getConfig('logger', winston.createLogger({
      defaultLevel,
      transports,
      exceptionHandlers: [new winston.transports.File({
        filename: _path.default.join(logsDir, 'exceptions.log'),
        maxsize: 5000000,
        maxFiles: 10,
        tailable: true,
        format: winston.format.combine(winston.format.timestamp(), winston.format.json())
      })]
    }));
    process.on('unhandledRejection', reason => logger.error(reason));
    this.export(_objectSpread({
      logger,
      log: (...args) => logger.log(...args)
    }, Object.keys(logger.levels).filter(level => logger.levels[level] <= logger.levels[logger.level]).reduce((acc, level) => _objectSpread({}, acc, {
      [level]: (message, meta) => logger[level](message, {
        meta
      })
    }), {}), {
      logError: this.logError
    }));

    if (printModules) {
      this.printModules();
    }

    if (printDependencyGraph) {
      this.printDependencyGraph();
    }

    if (printMiddlewares) {
      this.printMiddlewares();
    }

    if (logGqlErrors) {
      this.onModule('oors.graphql', 'error', error => {
        this.logError(error);
      });
    }
  }

  get logger() {
    return this.get('logger');
  }

  printModules() {
    const modulesTable = new _cliTable.default({
      head: ['Modules']
    });
    this.on('module:loaded', module => {
      modulesTable.push([module.name]);
    });
    this.once('after:setup', () => {
      console.log(modulesTable.toString());
    });
  }

  printDependencyGraph() {
    this.once('after:setup', () => {
      console.log(this.manager.expandedDependencyGraph);
    });
  }

  printMiddlewares() {
    if (!this.manager.has('oors.express')) {
      return;
    }

    this.once('after:setup', () => {
      const table = new _cliTable.default({
        head: ['Id', 'Path', 'Params']
      });
      this.manager.get('oors.express').middlewares.reject({
        enabled: false
      }).forEach(({
        path: mPath,
        id,
        params
      }) => {
        table.push([id, mPath || '/', typeof params !== 'undefined' ? (0, _util.inspect)(params) : 'N/A']);
      });
      console.log(table.toString());
    });
  }

}

exports.default = LoggerModule;
LoggerModule.validateConfig = (0, _easevalidation.validate)(_easevalidation.validators.isSchema({
  level: [_easevalidation.validators.isDefault('info'), _easevalidation.validators.isString()],
  printModules: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  printDependencyGraph: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  printMiddlewares: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()],
  logger: _easevalidation.validators.isAny(_easevalidation.validators.isUndefined(), _easevalidation.validators.isObject()),
  logsDir: [_easevalidation.validators.isRequired(), _easevalidation.validators.isString()],
  logGqlErrors: [_easevalidation.validators.isDefault(true), _easevalidation.validators.isBoolean()]
}));